@@ -13,7 +13,7 @@ from django.db.models.query import QuerySet, ValuesQuerySet, ValuesListQuerySet,
 from django.db.models.signals import post_save, post_delete, m2m_changed
 
 from cacheops.conf import model_profile, redis_client, handle_connection_failure
-from cacheops.utils import monkey_mix, dnf, conj_scheme, get_model_name, non_proxy
+from cacheops.utils import monkey_mix, dnf, conj_scheme, get_model_name, non_proxy, stamp_fields
 from cacheops.invalidation import cache_schemes, conj_cache_key, invalidate_obj, invalidate_model
 
 
@@ -134,6 +134,7 @@ def _stringify_query():
     """
     import simplejson as json
     from datetime import datetime, date
+    from django.db.models.expressions import ExpressionNode, F
     from django.db.models.fields import Field
     from django.db.models.sql.where import Constraint, WhereNode, ExtraWhere
     from django.db.models.sql import Query
@@ -142,11 +143,13 @@ def _stringify_query():
     from django.db.models.sql.expressions import SQLEvaluator
 
     attrs = {}
-    attrs[WhereNode] = ('connector', 'negated', 'children', 'subtree_parents')
+    attrs[WhereNode] = attrs[ExpressionNode] = ('connector', 'negated', 'children', 'subtree_parents')
+    attrs[SQLEvaluator] = ('expression',)
     attrs[ExtraWhere] = ('sqls', 'params')
     attrs[Aggregate] = ('source', 'is_summary', 'col', 'extra')
-    attrs[RawValue] = ('value')
+    attrs[RawValue] = ('value',)
     attrs[Date] = ('col', 'lookup_type')
+    attrs[F] = ('name',)
 
     q = Query(None)
     q_keys = q.__dict__.keys()
@@ -169,17 +172,15 @@ def _stringify_query():
             return (obj.alias, obj.col)
         elif isinstance(obj, Field):
             return (obj.model, obj.name)
-        elif isinstance(obj, QuerySet):
-            return (obj.__class__, obj.query)
         elif obj.__class__ in attrs:
             return (obj.__class__, [getattr(obj, attr) for attr in attrs[obj.__class__]])
+        elif isinstance(obj, QuerySet):
+            return (obj.__class__, obj.query)
         elif isinstance(obj, Aggregate):
             return (obj.__class__, [getattr(obj, attr) for attr in attrs[Aggregate]])
         elif isinstance(obj, Query):
             # for custom subclasses of Query
             return (obj.__class__, [getattr(obj, attr) for attr in attrs[Query]])
-        elif isinstance(obj, SQLEvaluator):
-            return (obj.__class__, obj.expression.__dict__.items())
         else:
             raise TypeError("Can't encode %s" % repr(obj))
 
@@ -230,6 +231,7 @@ class QuerySetMixin(object):
         """
         md5 = hashlib.md5()
         md5.update(str(self.__class__))
+        md5.update(stamp_fields(self.model)) # Protect from field list changes in model
         md5.update(stringify_query(self.query))
         if extra:
             md5.update(str(extra))
@@ -367,7 +369,11 @@ class QuerySetMixin(object):
                 except KeyError:
                     _local_get_cache[key] = self._no_monkey.get(self, *args, **kwargs)
                     return _local_get_cache[key]
-            elif 'fetch' in self._cacheops:
+                except TypeError:
+                    pass # If some arg is unhashable we can't save it to dict key,
+                         # we just skip local cache in that case
+
+            if 'fetch' in self._cacheops:
                 qs = self
             else:
                 qs = self._clone().cache()
